<?php
class AclFile
{
	// RD2 [fileID]=[user]|[group]|[mod]|[isChecked]|[isFreeFile]|
	var $_path="";
	var $_changed=false;
	var $_files;

	var $_POLY=0x8468;

	function AclFile($path)
	{
		$path=str_replace("\\","/",$path);
		if($path[strlen($path)-1] != "/")
		{
			$path=$path."/";
		}

		$this->_path=$path;
		$this->parse();
	}


	function applyChanges()
	{
		if(!$this->_changed || $this->_path=="")
			return;

		if(sizeof($this->_files)==0)
		{
			if(file_exists($this->_path.".raidenftpd.acl"))
			{
				unlink($this->_path.".raidenftpd.acl");
			}
			return;
		}

		if($fp=fopen($this->_path.".raidenftpd.acl","w"))
		{
			foreach($this->_files as $fileID=>$temp)
			{
				fwrite($fp,"RD2 $fileID=".$temp["user"]."|".
					$temp["group"]."|".
					$temp["mod"]."|".
					$temp["checked"]."|".
					$temp["free"]."|\r\n");
			}
			fclose($fp);
		}
	}

	function parse()
	{
		if($this->_path=="")
			return;

		if(file_exists($this->_path.".raidenftpd.acl"))
		{
			$lines=file($this->_path.".raidenftpd.acl");
		}
		else
		{
			$this_array=array();
			return;
		}

		foreach($lines as $line)
		{
			$line=substr($line,4);
			$elems=explode("=",$line);
			$fileID=array_shift($elems);
			$temp=explode("|",array_shift($elems));
			$this->_files[$fileID]=array();
			$this->_files[$fileID]["user"]=$temp[0];
			$this->_files[$fileID]["group"]=$temp[1];
			$this->_files[$fileID]["mod"]=$temp[2];
			$this->_files[$fileID]["checked"]=$temp[3];
			$this->_files[$fileID]["free"]=trim($temp[4]);
		}
	}

	function crc16($filename)
	{
		$filename=strtolower($filename);
		$nChars = strlen($filename);

		$crc = 0xffff;

		if ($nChars == 0) return (0);

		for ( $i = 0 ; $i < $nChars; $i++ )
		{  
			$data=0xff & ord($filename[$i]);
			for ($j=0; $j < 8; $j++ )
			{

				if (($crc & 0x0001) ^ ($data & 0x0001))
				{
					$crc = (($crc >> 1) ^ $this->_POLY);
				}
				else
				{
					$crc >>= 1;
				}
				$data >>= 1;
			}
		}
		$crc = $crc^0xFFFF;
		$crc = (($crc & 0xff) << 8) | (($crc >> 8) & 0xff);
		return $crc;
	}

	function checkSum($filename)
	{
		$filename=strtolower($filename);
		$len = strlen($filename);
		$filename[$len]="\0";

		if ($len == 0)
		{
			return 1;
		}
		$avg = 0;
		for($j = 0 ; $j < $len ; $j++)
		{
			$avg +=ord($filename[$j]);
		}
		return $avg%32768;
	}


	function getFileID($filename)
	{
		$crc16=$this->crc16($filename);
		$checksum=$this->checkSum($filename);

		return "".(($crc16 * pow(2,16))+$checksum);
	}

	function chmod($filename,$user,$group,$mod,$checked=null,$free=null)
	{
		if($user=="" || $group=="" || $mod=="" || $this->_path=="")
			return;

		$fileID=$this->getFileID($filename);

		$this->_changed=true;
		$this->_files[$fileID]["user"]=$user;
		$this->_files[$fileID]["group"]=$group;
		$this->_files[$fileID]["mod"]=$mod;

		if($checked!=null)
			$this->_files[$fileID]["checked"]=$checked;
		else if(!isset($this->_files[$fileID]["checked"]))
			$this->_files[$fileID]["checked"]=0;

		if($free!=null)
			$this->_files[$fileID]["free"]=$free;
		else if(!isset($this->_files[$fileID]["free"]))
			$this->_files[$fileID]["free"]=0;
	}

	function setCheckedFlag($filename, $checked=1)
	{
		if($this->_path=="")
			return;

		$fileID=$this->getFileID($filename);
		if(isset($this->_files[$fileID]))
		{
			$this->_changed=true;
			$this->_files[$fileID]["checked"]=$checked;
		}
	}

	function setFreeFlag($filename, $free=1)
	{
		if($this->_path=="")
			return;

		$fileID=$this->getFileID($filename);
		if(isset($this->_files[$fileID]))
		{
			$this->_changed=true;
			$this->_files[$fileID]["free"]=$free;
		}
	}

	function addFiles($files,$user,$group,$mod,$checked=null,$free=null)
	{
		if($this->_path=="")
			return;

		if(is_array($files))
		{
			foreach($files as $file)
			{
				$this->chmod($file,$user,$group,$mod,$checked,$free);
			}
		}
	}

	function delete($filename)
	{
		if($this->_path=="")
			return;

		$new_files=array();
		$fileID=$this->getFileID($filename);

		foreach($this->_files as $current_fileID=>$current_entry)
		{
			if($fileID==$current_fileID)
			{
				$this->_changed=true;
			}
			else
			{
				$new_files[$current_fileID]=$current_entry;
			}
		}
		$this->_files=$new_files;
	}

	function deleteFiles($files)
	{
		if($this->_path=="")
			return;

		if(is_array($files))
		{
			foreach($files as $file)
			{
				$this->delete($file);
			}
		}
	}

	function rename($oldname,$newname)
	{
		if($oldname=="" || $newname=="" || $this->_path=="")
			return;

		$oldFileID=$this->getFileID($oldname);
		$newFileID=$this->getFileID($newname);
		if(isset($this->_files[$oldFileID]))
		{
			$this->_files[$newFileID]=$this->_files[$oldFileID];
			$this->delete($oldname);
		}
	}
}
?>